(* util.sml
 *
 * Utility files
 *)

structure Util = struct
  
  local 
    fun curry f x = (fn y => f (x,y))
    fun appC f x y = app (curry f x) y
  in
    fun out os l = (appC TextIO.output os l; 
                    TextIO.output (os,"\n"))
  end

  fun comment os lines = 
    (out os [];
     out os ["(**********************************************************************"];
     out os [" *"];
     app (out os) (map (fn (s) => [" *  "^s]) lines);
     out os [" *"];
     out os [" **********************************************************************)"];
     out os [])
    
  fun small_comment os lines = 
    (out os ["(*"];
             app (out os) (map (fn (s) => [" * "^s]) lines);
             out os [" *)"])
    
  fun concatWith (s,[]) = ""
    | concatWith (s,[a]) = a
    | concatWith (s,a1::a2::r) = a1^s^concatWith(s,a2::r)
    
  fun mapI f l = let
    fun iterMapI f [] i = []
      | iterMapI f (x::xs) i = (f (x,i))::iterMapI f xs (i+1)
  in
    iterMapI f l 0
  end

  fun appI f l = let
    fun iterAppI f [] i = ()
      | iterAppI f (x::xs) i = (f (x,i); iterAppI f xs (i+1))
  in
    iterAppI f l 0
  end

  fun generated_message os = let
    val d = Date.fromTimeLocal (Time.now ())
  in
    comment os ["This file was automatically generated",
                "("^(Date.toString d)^")"]
  end

  fun withOutputFile (file, f) = let
    val os = TextIO.openOut (file)
    val _ = generated_message os
    val r = f (os)
  in
    TextIO.closeOut (os);
    r
  end


  fun withInputFile (file, f) = let
    val is = TextIO.openIn (file)
    val r = f (is)
  in
    TextIO.closeIn (is);
    r
  end

  fun replace_extension (f,e) = let
    val {base,ext} = OS.Path.splitBaseExt (f)
  in
    OS.Path.joinBaseExt {base=base,ext=SOME(e)}
  end

  fun append_and_replace_extension (f,a,e) = let
    val {base,ext} = OS.Path.splitBaseExt (f)
  in
    OS.Path.joinBaseExt {base=base^a,ext=SOME(e)}
  end

end
